@@ -115,7 +115,7 @@ entry_get_string (GConfEntry *entry)
 static gboolean
 parse_binding (Binding *binding)
 {
-        gboolean success;
+        EggParseError ret;
 
         g_return_val_if_fail (binding != NULL, FALSE);
 
@@ -130,15 +130,15 @@ parse_binding (Binding *binding)
                 return FALSE;
         }
 
-        success = egg_accelerator_parse_virtual (binding->binding_str,
-                                                 &binding->key.keysym,
-                                                 &binding->key.keycodes,
-                                                 &binding->key.state);
+        ret = egg_accelerator_parse_virtual (binding->binding_str,
+                                             &binding->key.keysym,
+                                             &binding->key.keycodes,
+                                             &binding->key.state);
 
-        if (!success)
-            g_warning (_("Key binding (%s) is invalid"), binding->gconf_key);
+        if (ret != EGG_PARSE_ERROR_NONE)
+            g_warning (_("Key binding (%s) is invalid (%d)"), binding->gconf_key, ret);
 
-        return success;
+        return (ret == EGG_PARSE_ERROR_NONE);
 }
 
 static gint
@@ -315,7 +315,7 @@ binding_unregister_keys (GsdKeybindingsManager *manager)
 
         if (need_flush)
                 gdk_flush ();
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static void
@@ -412,7 +412,7 @@ get_exec_environment (XEvent *xevent)
         int        i;
         int        display_index = -1;
         GdkScreen *screen = NULL;
-        GdkWindow *window = gdk_xid_table_lookup (xevent->xkey.root);
+        GdkWindow *window = gdk_x11_window_lookup_for_display (gdk_display_get_default (), xevent->xkey.root);
 
         if (window) {
                 screen = gdk_window_get_screen (window);
@@ -657,10 +657,6 @@ gsd_keybindings_manager_set_property (GObject        *object,
                                const GValue   *value,
                                GParamSpec     *pspec)
 {
-        GsdKeybindingsManager *self;
-
-        self = GSD_KEYBINDINGS_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -674,10 +670,6 @@ gsd_keybindings_manager_get_property (GObject        *object,
                                GValue         *value,
                                GParamSpec     *pspec)
 {
-        GsdKeybindingsManager *self;
-
-        self = GSD_KEYBINDINGS_MANAGER (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -691,9 +683,6 @@ gsd_keybindings_manager_constructor (GType                  type,
                               GObjectConstructParam *construct_properties)
 {
         GsdKeybindingsManager      *keybindings_manager;
-        GsdKeybindingsManagerClass *klass;
-
-        klass = GSD_KEYBINDINGS_MANAGER_CLASS (g_type_class_peek (GSD_TYPE_KEYBINDINGS_MANAGER));
 
         keybindings_manager = GSD_KEYBINDINGS_MANAGER (G_OBJECT_CLASS (gsd_keybindings_manager_parent_class)->constructor (type,
                                                                                                       n_construct_properties,
@@ -705,10 +694,6 @@ gsd_keybindings_manager_constructor (GType                  type,
 static void
 gsd_keybindings_manager_dispose (GObject *object)
 {
-        GsdKeybindingsManager *keybindings_manager;
-
-        keybindings_manager = GSD_KEYBINDINGS_MANAGER (object);
-
         G_OBJECT_CLASS (gsd_keybindings_manager_parent_class)->dispose (object);
 }
 
